/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "outlook/ContactBuilder.h"

//#include "base/quoted-printable.h"
//#include "base/util/utils.h"


/*
* Method to create a SIF String from a contact Vobject. 
* It get every property name and value and build the XML structure.
*
* @param contactStringItem : the string that will contain the xml structure
* @param vo                : the contact object that contain every value
*
*/
void VObjectContact2SIF(string &contactStringItem, VObject *vo) {
	
	int businessTel  = 0;
	int homeTel		 = 0;
	int email		 = 0;
	wchar_t* name    = NULL;
    wchar_t* element = NULL;
    
	contactStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	contactStringItem = contactStringItem + TEXT("<contact>");


	// Convert each property:
	// ----------------------
    for(int i = 0; i < vo->propertiesCount(); i++) {

		name    = vo->getProperty(i)->getName();
        element = vo->getProperty(i)->getValue();

		// Empty elements are skipped.
        if( element != NULL && wcscmp(element, TEXT("")) ) {
			
			if(vo->getProperty(i)->equalsEncoding(TEXT("QUOTED-PRINTABLE")) && 
				!wcscmp(vo->getProperty(i)->getParameterValue(TEXT("CHARSET")), TEXT("UTF-8"))) {

				wchar_t* decodedVal = decodeQpUTF8VProp(element);
				if(decodedVal) {
					vo->getProperty(i)->setValue(decodedVal);
				}				
				//else
				//if decoded fails we left the original value
			}

			if(!wcscmp(name, TEXT("N"))) {
				if(element = vo->getProperty(i)->getPropComponent(1))
					contactStringItem  = contactStringItem + TEXT("<LastName>") + element + TEXT("</LastName>");
				if(element = vo->getProperty(i)->getPropComponent(2))
					contactStringItem  = contactStringItem + TEXT("<FirstName>") + element + TEXT("</FirstName>");
				if(element = vo->getProperty(i)->getPropComponent(3))
					contactStringItem  = contactStringItem + TEXT("<MiddleName>") + element + TEXT("</MiddleName>");
				if(element = vo->getProperty(i)->getPropComponent(4))
					contactStringItem  = contactStringItem + TEXT("<Title>") + element + TEXT("</Title>");
				if(element = vo->getProperty(i)->getPropComponent(5))
					contactStringItem  = contactStringItem + TEXT("<Suffix>") + element + TEXT("</Suffix>");
			}


			else if(!wcscmp(name, TEXT("BDAY")))
				contactStringItem  = contactStringItem + TEXT("<Birthday>") + element + TEXT("</Birthday>");
			
			else if(!wcscmp(name, TEXT("TEL")) && 
				vo->getProperty(i)->containsParameter(TEXT("WORK"))) {
				if (vo->getProperty(i)->containsParameter(TEXT("FAX")))
					contactStringItem  = contactStringItem + TEXT("<BusinessFaxNumber>") + element + TEXT("</BusinessFaxNumber>");
				else if (vo->getProperty(i)->containsParameter(TEXT("CELL")))
					;
				else 
					if(businessTel == 0) {
						contactStringItem  = contactStringItem + TEXT("<BusinessTelephoneNumber>") + element + TEXT("</BusinessTelephoneNumber>");
						businessTel++;
					}
					else{
						contactStringItem  = contactStringItem + TEXT("<Business2TelephoneNumber>") + element + TEXT("</Business2TelephoneNumber>");
					}
			}
			
			else if(!wcscmp(name, TEXT("ORG"))) {
				element = vo->getProperty(i)->getPropComponent(1);
				contactStringItem  = contactStringItem + TEXT("<CompanyName>") + element + TEXT("</CompanyName>");
			}
			else if(!wcscmp(name, TEXT("EMAIL"))) {
				if(email == 0) {
					contactStringItem  = contactStringItem + TEXT("<Email1Address>") + element + TEXT("</Email1Address>");
					email++;
				}
				else if(email == 1) {
					email++;
					contactStringItem  = contactStringItem + TEXT("<Email2Address>") + element + TEXT("</Email2Address>");
				}
				else
					contactStringItem  = contactStringItem + TEXT("<Email3Address>") + element + TEXT("</Email3Address>");
			}

			else if(!wcscmp(name, TEXT("TEL")) &&
				vo->getProperty(i)->containsParameter(TEXT("CELL")))
				contactStringItem  = contactStringItem + TEXT("<MobileTelephoneNumber>") + element + TEXT("</MobileTelephoneNumber>");

			else if(!wcscmp(name, TEXT("TEL")) && 
				vo->getProperty(i)->containsParameter(TEXT("PAGER")))
				contactStringItem  = contactStringItem + TEXT("<PagerNumber>") + element + TEXT("</PagerNumber>");
			
			else if(!wcscmp(name, TEXT("TITLE"))) 
				contactStringItem  = contactStringItem + TEXT("<JobTitle>") + element + TEXT("</JobTitle>");

			else if(!wcscmp(name, TEXT("TEL")) &&
				vo->getProperty(i)->containsParameter(TEXT("HOME"))) {
				if(vo->getProperty(i)->containsParameter(TEXT("FAX")))
					contactStringItem  = contactStringItem + TEXT("<HomeFaxNumber>") + element + TEXT("</HomeFaxNumber>");
				else if(vo->getProperty(i)->containsParameter(TEXT("CELL")))
					;
				else if(homeTel == 0) {
					contactStringItem  = contactStringItem + TEXT("<HomeTelephoneNumber>") + element + TEXT("</HomeTelephoneNumber>");
					homeTel++;
				}
				else
					contactStringItem  = contactStringItem + TEXT("<Home2TelephoneNumber>") + element + TEXT("</Home2TelephoneNumber>");
			}
			
			else if(!wcscmp(name, TEXT("TEL")) && 
					vo->getProperty(i)->containsParameter(TEXT("CAR")))
				contactStringItem  = contactStringItem + TEXT("<CarTelephoneNumber>") + element + TEXT("</CarTelephoneNumber>");

			else if(!wcscmp(name, TEXT("TEL")) && 
					vo->getProperty(i)->containsParameter(TEXT("VOICE")))
				contactStringItem  = contactStringItem + TEXT("<OtherTelephoneNumber>") + element + TEXT("</OtherTelephoneNumber>");

			else if(!wcscmp(name, TEXT("CATEGORIES")))
				contactStringItem  = contactStringItem + TEXT("<Categories>") + element + TEXT("</Categories>");

			else if(!wcscmp(name, TEXT("URL")))
				contactStringItem  = contactStringItem + TEXT("<WebPage>") + element + TEXT("</WebPage>");

			else if(!wcscmp(name, TEXT("NOTE")))
				contactStringItem  = contactStringItem + TEXT("<Body>") + element + TEXT("</Body>");
			
			else if(!wcscmp(name, TEXT("ADR"))) 
				if(vo->getProperty(i)->containsParameter(TEXT("HOME"))) {
					if(element = vo->getProperty(i)->getPropComponent(3))
						contactStringItem  = contactStringItem + TEXT("<HomeAddressStreet>") + element + TEXT("</HomeAddressStreet>");
					if(element = vo->getProperty(i)->getPropComponent(4))
						contactStringItem  = contactStringItem + TEXT("<HomeAddressCity>") + element + TEXT("</HomeAddressCity>");
					if(element = vo->getProperty(i)->getPropComponent(5))
						contactStringItem  = contactStringItem + TEXT("<HomeAddressState>") + element + TEXT("</HomeAddressState>");
					if(element = vo->getProperty(i)->getPropComponent(6))
						contactStringItem  = contactStringItem + TEXT("<HomeAddressPostalCode>") + element + TEXT("</HomeAddressPostalCode>");
					if(element = vo->getProperty(i)->getPropComponent(7))
						contactStringItem  = contactStringItem + TEXT("<HomeAddressCountry>") + element + TEXT("</HomeAddressCountry>");
				}
				else if(vo->getProperty(i)->containsParameter(TEXT("WORK"))) {
					if(element = vo->getProperty(i)->getPropComponent(3))
						contactStringItem  = contactStringItem + TEXT("<BusinessAddressStreet>") + element + TEXT("</BusinessAddressStreet>");
					if(element = vo->getProperty(i)->getPropComponent(4))
						contactStringItem  = contactStringItem + TEXT("<BusinessAddressCity>") + element + TEXT("</BusinessAddressCity>");
					if(element = vo->getProperty(i)->getPropComponent(5))
						contactStringItem  = contactStringItem + TEXT("<BusinessAddressState>") + element + TEXT("</BusinessAddressState>");
					if(element = vo->getProperty(i)->getPropComponent(6))
						contactStringItem  = contactStringItem + TEXT("<BusinessAddressPostalCode>") + element + TEXT("</BusinessAddressPostalCode>");
					if(element = vo->getProperty(i)->getPropComponent(7))
						contactStringItem  = contactStringItem + TEXT("<BusinessAddressCountry>") + element + TEXT("</BusinessAddressCountry>");
				}
				else {
					if(element = vo->getProperty(i)->getPropComponent(3))
						contactStringItem  = contactStringItem + TEXT("<OtherAddressStreet>") + element + TEXT("</OtherAddressStreet>");
					if(element = vo->getProperty(i)->getPropComponent(4))
						contactStringItem  = contactStringItem + TEXT("<OtherAddressCity>") + element + TEXT("</OtherAddressCity>");
					if(element = vo->getProperty(i)->getPropComponent(5))
						contactStringItem  = contactStringItem + TEXT("<OtherAddressState>") + element + TEXT("</OtherAddressState>");
					if(element = vo->getProperty(i)->getPropComponent(6))
						contactStringItem  = contactStringItem + TEXT("<OtherAddressPostalCode>") + element + TEXT("</OtherAddressPostalCode>");
					if(element = vo->getProperty(i)->getPropComponent(7))
						contactStringItem  = contactStringItem + TEXT("<OtherAddressCountry>") + element + TEXT("</OtherAddressCountry>");
				}
		}
    }

	contactStringItem = contactStringItem +  TEXT("</contact>");

}






/*
* Method that complete a contact object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the VObject.
* Note:
* The returned VObject 'vo' is dinamically allocated inside this method, must be freed by the caller.
*
* @param vo       : the contact object that contain every value
* @param ptrData  : the variable that contain the xml structure
*/
VObject*  SIF2VObjectContact(wchar_t * ptrData) {
	
	VObject* vo = new VObject();

	wchar_t* element = NULL;
	wchar_t name[100];
	wchar_t fName[100];
	DATE date = NULL;
	VProperty* vprop = NULL;
	VProperty* vp = NULL;


	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VCARD")));
	vo->addProperty(new VProperty(TEXT("VERSION"), TEXT("2.1")));

	// ------- Name -------
	wsprintf(name, TEXT(""));
	wsprintf(fName, TEXT(""));

	element = getElementContent(ptrData, TEXT ("LastName"), NULL);
	if(wcscmp(element, TEXT(""))) {
		wcscat(name, element);
		wcscat(fName, element);
		wcscat(fName, TEXT(","));	
	}
    wcscat(name, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("FirstName"), NULL);
    if(wcscmp(element, TEXT(""))) {
		wcscat(name, element);
		wcscat(fName, element);
	}
    wcscat(name, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("MiddleName"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(name, element);
    wcscat(name, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Title"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(name, element);
    wcscat(name, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Suffix"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(name, element);
	delete [] element; element = NULL;
 

	vprop = buildQpVProp(TEXT("N"), name);
	vo->addProperty(vprop);
	delete vprop;

	vprop = buildQpVProp(TEXT("FN"), fName);
	vo->addProperty(vprop);
	delete vprop;



	element = getElementContent(ptrData, TEXT ("Birthday"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vo->addProperty(new VProperty(TEXT("BDAY"), element));
		delete [] element; element = NULL;
	}

	//Anniversary has no corespondent in the VCARD

	element = getElementContent(ptrData, TEXT ("BusinessFaxNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("WORK"), NULL);
		vp->addParameter(TEXT("FAX"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("CompanyName"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("ORG"), element);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	//Department has no corespondent in the VCARD
	
	element = getElementContent(ptrData, TEXT ("Email1Address"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("EMAIL"), element);
		vprop->addParameter(TEXT("INTERNET"), NULL);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Email2Address"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("EMAIL"), element);
		vprop->addParameter(TEXT("INTERNET"), NULL);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Email3Address"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("EMAIL"), element);
		vprop->addParameter(TEXT("INTERNET"), NULL);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 	
	element = getElementContent(ptrData, TEXT ("MobileTelephoneNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("CELL"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	//OfficeLocation has no corespondent in the VCARD

	element = getElementContent(ptrData, TEXT ("PagerNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("PAGER"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("BusinessTelephoneNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("WORK"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("JobTitle"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("TITLE"), element);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("HomeTelephoneNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("HOME"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	//Spouse has no corespondent in the VCARD

	element = getElementContent(ptrData, TEXT ("Home2TelephoneNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("HOME"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("CarTelephoneNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("CAR"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("HomeFaxNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("HOME"), NULL);
		vp->addParameter(TEXT("FAX"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("OtherTelephoneNumber"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("VOICE"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;

	// AssistantName X 
	// AssistantTelephoneNumber X 
	//Children has no corespondent in the VCARD
	
	element = getElementContent(ptrData, TEXT ("Categories"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("CATEGORIES"), element);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("WebPage"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("URL"), element);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Business2TelephoneNumber"), NULL);
    if(wcscmp(element, TEXT(""))) {
		vp = new VProperty(TEXT("TEL"), element);
		vp->addParameter(TEXT("WORK"), NULL);
		vo->addProperty(vp);
		delete vp; vp = NULL;
	}
	delete [] element; element = NULL;
 
	//RadioTelephoneNumber has no corespondent in the VCARD
	//Yomi properties have no corespondent in the VCARD
	

	// ----- Address HOME -----
	wchar_t address[200];
	wsprintf(address, TEXT(";;"));
	element = getElementContent(ptrData, TEXT ("HomeAddressStreet"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;

	element = getElementContent(ptrData, TEXT ("HomeAddressCity"), NULL);
	if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("HomeAddressState"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("HomeAddressPostalCode"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("HomeAddressCountry"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	if(wcscmp(address, TEXT(";;;;;;"))) {
		vprop = buildQpVProp(TEXT("ADR"), address);
		vprop->addParameter(TEXT("HOME"), NULL);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 

	// ----- Address -----
	wsprintf(address, TEXT(";;"));
	element = getElementContent(ptrData, TEXT ("OtherAddressStreet"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("OtherAddressCity"), NULL);
	if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("OtherAddressState"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("OtherAddressPostalCode"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("OtherAddressCountry"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	if(wcscmp(address, TEXT(";;;;;;"))) {
		vprop = buildQpVProp(TEXT("ADR"), address);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 
	
	// ----- Address WORK -----
	wsprintf(address, TEXT(";;"));
	element = getElementContent(ptrData, TEXT ("BusinessAddressStreet"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("BusinessAddressCity"), NULL);
	if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("BusinessAddressState"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("BusinessAddressPostalCode"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	wcscat(address, TEXT(";"));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("BusinessAddressCountry"), NULL);
    if(wcscmp(element, TEXT("")))
		wcscat(address, element);
	if(wcscmp(address, TEXT(";;;;;;"))) {
		vprop = buildQpVProp(TEXT("ADR"), address);
		vprop->addParameter(TEXT("WORK"), NULL);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 

	//FileAs has no corespondent in the VCARD

	element = getElementContent(ptrData, TEXT ("Body"), NULL);
	if(wcscmp(element, TEXT(""))) {
		vprop = buildQpVProp(TEXT("NOTE"), element);
		vo->addProperty(vprop);
		delete vprop;
	}
	delete [] element; element = NULL;
 

	vo->addProperty(new VProperty(TEXT("END"), TEXT("VCARD")));

    return vo;
}




VProperty *buildQpVProp(wchar_t* name, BSTR welement) {
	
	char* element;
	element = toMultibyte(welement);

	VProperty* vp = new VProperty(name, NULL);
	if(qp_isNeed(element)) {
		char* fAppend = new char[200];
		strcpy(fAppend, "");
		bool ret = addQpUTFString(element, fAppend, 0);
		if(ret) {
			int len = strlen(fAppend) + 1;
			wchar_t* encoded = new wchar_t[len]; 
			mbstowcs(encoded, fAppend, len);
			vp->setValue(encoded);
			vp->addParameter(TEXT("ENCODING"), TEXT("QUOTED-PRINTABLE"));
			vp->addParameter(TEXT("CHARSET"), TEXT("UTF-8"));
		}
		else
			vp->setValue(welement);
		delete [] fAppend;
	}
	else
		vp->setValue(welement);

	if (element) 
		delete [] element;

	return vp;
}