/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "outlook/TaskBuilder.h"

/*
* Method to create a SIF String from a task Vobject. 
* It get every property name and value and build the XML structure.
*
* @param contactStringItem : the string that will contain the xml structure
* @param vo                : the task object that contain every value
*
*/
void VObjectTask2SIF (string &taskStringItem, VObject *vo) {

	wchar_t* element = NULL;
	wchar_t tempbuf [DIM_LOCAL_TEMP];
	DATE startdate   = NULL;
	long importance;


	taskStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    taskStringItem = taskStringItem + TEXT("<task>");


    if(vo->containsProperty(TEXT("SUMMARY")) && vo->getProperty(TEXT("SUMMARY"))->getValue()) {
        element = vo->getProperty(TEXT("SUMMARY"))->getValue();
		taskStringItem  = taskStringItem + TEXT("<Subject>") + element + TEXT("</Subject>");
    }

    if(vo->containsProperty(TEXT("DESCRIPTION")) && vo->getProperty(TEXT("DESCRIPTION"))->getValue()) {
        element = vo->getProperty(TEXT("DESCRIPTION"))->getValue();
		taskStringItem  = taskStringItem + TEXT("<Body>") + element + TEXT("</Body>");
    }
	
	if(vo->containsProperty(TEXT("DTSTART")) && vo->getProperty(TEXT("DTSTART"))->getValue()) {
		element = vo->getProperty(TEXT("DTSTART"))->getValue();
		taskStringItem  = taskStringItem + TEXT("<StartDate>") + element + TEXT("</StartDate>");
		systemTimeToDouble(element, &startdate, NULL);
	}

	if(vo->containsProperty(TEXT("DUE")) && vo->getProperty(TEXT("DUE"))->getValue()) {
		element = vo->getProperty(TEXT("DUE"))->getValue();
		taskStringItem  = taskStringItem + TEXT("<DueDate>") + element + TEXT("</DueDate>");
	}

    if(vo->containsProperty(TEXT("CATEGORIES")) && vo->getProperty(TEXT("CATEGORIES"))->getValue()) {
        element = vo->getProperty(TEXT("CATEGORIES"))->getValue();
		taskStringItem  = taskStringItem + TEXT("<Categories>") + element + TEXT("</Categories>");
    }


	if(vo->containsProperty(TEXT("CLASS")) && vo->getProperty(TEXT("CLASS"))->getValue()) {
        element = vo->getProperty(TEXT("CLASS"))->getValue();
		if(!wcscmp(element, TEXT("PRIVATE")) || !wcscmp(element, TEXT("CONFIDENTIAL")))
			wsprintf(tempbuf, TEXT("%i"), olPrivate);		// Private = 2
		else
			wsprintf(tempbuf, TEXT("%i"), olNormal);		// Normal  = 0
	}
	else
		wsprintf(tempbuf, TEXT("%i"), olNormal);			// Normal  = 0

	taskStringItem  = taskStringItem + TEXT("<Sensitivity>") + tempbuf + TEXT("</Sensitivity>");


	//alarm
	//The value consists of: Run Time, Snooze Time; Repeat Count, Audio Content
	if(vo->containsProperty(TEXT("AALARM")) && vo->getProperty(TEXT("AALARM"))->getValue()) {
		
		wsprintf(tempbuf, TEXT("%i"), 1);	// TRUE
		taskStringItem  = taskStringItem + TEXT("<ReminderSet>") + tempbuf + TEXT("</ReminderSet>");
		
		element = vo->getProperty(TEXT("AALARM"))->getPropComponent(1);
		taskStringItem  = taskStringItem + TEXT("<ReminderTime>") + element + TEXT("</ReminderTime>");
	}
	
	if(vo->containsProperty(TEXT("COMPLETED")) && vo->getProperty(TEXT("COMPLETED"))->getValue()) {
		wsprintf(tempbuf, TEXT("%i"), 1);	// TRUE
		taskStringItem  = taskStringItem + TEXT("<Complete>") + tempbuf + TEXT("</Complete>");
	}
	
	if(vo->containsProperty(TEXT("PRIORITY")) && vo->getProperty(TEXT("PRIORITY"))->getValue()) {
        element = vo->getProperty(TEXT("PRIORITY"))->getValue();

		if(!wcscmp(element, TEXT("1")))
			importance = olImportanceHigh;
		else if(!wcscmp(element, TEXT("2")))
			importance = olImportanceNormal;
		else if(!wcscmp(element, TEXT("3")))
			importance = olImportanceLow;

		wsprintf(tempbuf, TEXT("%i"), importance);
        taskStringItem  = taskStringItem + TEXT("<Importance>") + tempbuf + TEXT("</Importance>");
	}
	

	//recurrence
	///////////////////////////////////////////////////////////////////////////////////
	if((vo->containsProperty(TEXT("RRULE")) && vo->getProperty(TEXT("RRULE"))->getValue())) {

		wsprintf(tempbuf, TEXT("%i"), 1);	// TRUE
		taskStringItem  = taskStringItem + TEXT("<IsRecurring>") + tempbuf + TEXT("</IsRecurring>");

		string recurrenceString = string();
		bool ret = convertRruleToSIF(recurrenceString, vo->getProperty(TEXT("RRULE"))->getValue(), startdate);

		if(ret)
			taskStringItem = taskStringItem + recurrenceString;
	}

	else {
		wsprintf(tempbuf, TEXT("%i"), 0);	// FALSE
		taskStringItem  = taskStringItem + TEXT("<IsRecurring>") + tempbuf + TEXT("</IsRecurring>");
	}
	////////////////////////////////////////////////////////////////////////////////////


	taskStringItem = taskStringItem + TEXT("</task>");
}



/*
* Method that complete a task object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the VObject.
* Note:
* The returned VObject 'vo' is dinamically allocated inside this method, must be freed by the caller.
*
* @param ptrData  : the variable that contain the xml structure
* @return vo      : the task object that contain every value
*/
VObject* SIF2VObjectTask (wchar_t * ptrData) {

	VObject* vo = new VObject();

	wchar_t* element   = NULL;
	wchar_t* dateValue = NULL;
	

	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VCALENDAR")));
	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VTODO")));
	
	element = getElementContent(ptrData, TEXT ("Subject"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("SUMMARY"), element));
	delete [] element; element = NULL;

	element = getElementContent(ptrData, TEXT ("Body"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("DESCRIPTION"), element));
	delete [] element; element = NULL;
 

	element = getElementContent(ptrData, TEXT ("StartDate"), NULL);
	vo->addProperty(new VProperty(TEXT("DTSTART"), element));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("DueDate"), NULL);
	vo->addProperty(new VProperty(TEXT("DUE"), element));
	delete [] element; element = NULL;
 

	BOOL completed;
	element = getElementContent(ptrData, TEXT ("Complete"), NULL);
	completed = _wtoi(element);
	delete [] element; element = NULL;
 
	if(completed) {
		element = getElementContent(ptrData, TEXT ("DateCompleted"), NULL);
		vo->addProperty(new VProperty(TEXT("COMPLETED"), element));
		delete [] element; element = NULL;
	}


	element = getElementContent(ptrData, TEXT ("Categories"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("CATEGORIES"), element));
	delete [] element; element = NULL;
 

	long sensitivity;
	element = getElementContent(ptrData, TEXT ("Sensitivity"), NULL);
	sensitivity = _wtoi(element);
	delete [] element; element = NULL;

	if(sensitivity == olPrivate)
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PRIVATE")));
	else
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PUBLIC")));


	BOOL bReminder;
	element = getElementContent(ptrData, TEXT ("ReminderSet"), NULL);
	bReminder = _wtoi(element);
	delete [] element; element = NULL;
 
	if(bReminder == TRUE) {
		DATE reminder;
		element = getElementContent(ptrData, TEXT ("ReminderTime"), NULL);
		reminder = _wtoi(element);

		doubleToSystemTime(dateValue, reminder);
		
		wcscat(dateValue, TEXT(";;0;reminder"));
		vo->addProperty(new VProperty(TEXT("AALARM"), dateValue));
		delete [] dateValue; dateValue = NULL;	
		delete [] element; element = NULL;
	}


	long importance;
	element = getElementContent(ptrData, TEXT ("Importance"), NULL);
	importance = _wtoi(element);
	delete [] element; element = NULL;
 
	if(importance == olImportanceLow)
		vo->addProperty(new VProperty(TEXT("PRIORITY"), TEXT("3")));
	else if(importance == olImportanceNormal)
		vo->addProperty(new VProperty(TEXT("PRIORITY"), TEXT("2")));
	else if(importance == olImportanceHigh)
		vo->addProperty(new VProperty(TEXT("PRIORITY"), TEXT("1")));


	//recurrence
	///////////////////////////////////////////////////////////////////////
	BOOL isRec;
	element = getElementContent(ptrData, TEXT ("IsRecurring"), NULL);
	isRec = _wtoi(element);
	delete [] element; element = NULL;

	if(isRec) {
		wchar_t* recRule = NULL;

		recRule = convertSIFToRrule(ptrData);

		if(recRule)
			vo->addProperty(new VProperty(TEXT("RRULE"), recRule));
		
		delete [] recRule; recRule = NULL;
	}	
	///////////////////////////////////////////////////////////////////////


	vo->addProperty(new VProperty(TEXT("END"), TEXT("VTODO")));
	vo->addProperty(new VProperty(TEXT("END"), TEXT("VCALENDAR")));

    return vo;
}


